import datetime
import logging
import os

LOG_LEVEL = 'debug'
LOG_STREAM_LEVEL = 'debug'
LOG_FILE_LEVEL = 'info'
BASE_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
LOG_FOLDER = os.path.join(BASE_PATH, 'logs')
LOG_FILE_NAME = os.path.join(LOG_FOLDER, datetime.datetime.now().strftime('%Y-%m-%d') + '.log')


class LoggerHandler:
    """
    about log
    """
    _logger_level = {
        'debug': logging.DEBUG,
        'info': logging.INFO,
        'warning': logging.WARNING,
        'error': logging.ERROR,
        'critical': logging.CRITICAL
    }

    def __init__(self, log_name, file_name, logger_level, stream_level='info', file_level='warning'):
        self.log_name = log_name
        self.file_name = file_name
        self.logger_level = self._logger_level.get(logger_level, 'debug')
        self.stream_level = self._logger_level.get(stream_level, 'info')
        self.file_level = self._logger_level.get(file_level, 'warning')
        self.logger = logging.getLogger(self.log_name)
        self.logger.setLevel(self.logger_level)
        if not self.logger.handlers:
            f_stream = logging.StreamHandler()
            f_file = logging.FileHandler(self.file_name, encoding='utf-8')
            f_stream.setLevel(self.stream_level)
            f_file.setLevel(self.file_level)

            # set log output stream, or"%(asctime)s %(name)s %(levelname)s %(message)s", see more setting in "formater"
            formatter = logging.Formatter("%(asctime)s %(levelname)s %(message)s")
            f_stream.setFormatter(formatter)
            f_file.setFormatter(formatter)
            self.logger.addHandler(f_stream)
            self.logger.addHandler(f_file)

    @property
    def get_logger(self):
        return self.logger


def logger(log_name="DEFAULT"):
    if not os.path.exists(LOG_FOLDER):
        os.mkdir(LOG_FOLDER)
    return LoggerHandler(
        log_name=log_name,
        logger_level=LOG_LEVEL,
        file_name=LOG_FILE_NAME,
        stream_level=LOG_STREAM_LEVEL,
        file_level=LOG_FILE_LEVEL
    ).get_logger


# if __name__ == '__main__':
#     logger().info("somethin...")
