import platform
import random
from bottle import template
import webbrowser
import sys
import datetime
import os
from apist.html_ import REPORT_Table_Detail, REPORT_Table_Data, EXPORT_Detali_text, \
    REPORT_Title_Lable, REPORT_TMPL_Lable, html_

cases = {}
report_save_path = os.path.abspath(__file__)


def generate(all_case: list = '', right_case: list = '', error_case: list = '',
             untreaded_cases: list = '', report_file_path='', waste_time=None):
    """
    generate report by all these cases
    """
    global cases
    if not all_case or (not right_case and not error_case and not untreaded_cases):
        raise ValueError("No cases report")

    cases = {
        'all_cases': all_case,
        'right_cases': right_case,
        'error_cases': error_case,
        'untreaded_cases': untreaded_cases
    }

    # generate report
    html_report_object = HTML_REPORT(report_cases=cases, delta=waste_time)
    html = html_report_object.gen_html_report(html_)
    dir_ = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    if not report_file_path:
        file_name = datetime.datetime.now().strftime("%Y%m%d_%H%M%S") + ".html"
        report_file_path = os.path.join(dir_, "reports", file_name)
    with open(report_file_path, 'wb') as f:
        f.write(html.encode('utf-8'))

    fixed_path = os.path.join(dir_, "report.html")
    with open(fixed_path, 'wb') as f:
        f.write(html.encode('utf-8'))

    # report open, workflow(linux) don't need open it
    if platform.system().lower() in ["windows", "macos"]:
        webbrowser.open(fixed_path)


class HTML_REPORT:
    def __init__(self, html_title: str = '测试报告', pie_theme: str = '数据统计', test_info: list = None,
                 report_table_title: list = None, report_cases: dict = None, delta=None):
        """
        @parameter html_title:HTML标题
        @parameter pie_theme:饼图名称
        @parameter test_info:饼图数据统计，长度 = 6
        @parameter report_table_title:接口表格标题,列表类型，可以自定义
        @parameter report_cases:接口测试结果记录，字典格式且key不能变
        """
        self.html_title = html_title
        self.pie_theme = pie_theme
        if not pie_theme or not isinstance(html_title, str):
            print('invalid html title or pie name')
            sys.exit(0)

        if not report_table_title:
            self.report_table_title = ['用例id', '接口说明', 'URL', '状态码', '期望值', '测试结果', '详情']
        elif not isinstance(report_table_title, list):
            print('invalid type of report_table_title, it must be list')
            sys.exit(0)

        now1 = datetime.datetime.now()
        time1 = now1.strftime("%Y-%m-%d %H:%M:%S")
        if not delta:
            m = random.randint(0, 3)
            s = random.randint(1, 59)
            delta = datetime.timedelta(seconds=s, minutes=m)
        now2 = now1 - delta
        time2 = now2.strftime("%Y-%m-%d %H:%M:%S")
        d_s = delta.seconds
        d_s = d_s if d_s else 1
        m = d_s // 60
        s = d_s % 60
        waste = str(m) + "分" + str(s) + "秒"

        len_all = len(report_cases)
        len_r = len(report_cases['right_cases'])
        len_e = len(report_cases['error_cases'])
        len_u = len(report_cases['untreaded_cases'])

        if not test_info:
            test_info = [
                time2, time1, waste,
                len(report_cases['all_cases']),
                len_r + len_e,
                len_u
            ]
            self.test_info = test_info
        elif not isinstance(test_info, list):
            print('invalid type of test_info, it must be list')
            sys.exit(0)

        if isinstance(report_cases, dict) and len_all > 0:
            self.report_cases = report_cases
            self.pie_sum_number = {
                'right_sum': len_r if len_r else 0.01,
                'error_sum': len_e if len_e else 0.01,
                'untreated_sum': len_u if len_u else 0.01
            }
        else:
            print('invalid type of report_cases, it must be list')
            sys.exit(0)

    def packaged_cases(self, export_label_title, data_key, class_name, pannel_num):
        cases_num = 1
        cases_packaged = ''
        cases_packaged = ''.join([cases_packaged,
                                  '                   <tr class="{}">\n'.format(class_name),
                                  export_label_title,
                                  '                   </tr>\n'])
        for data_case in self.report_cases[data_key]:
            detail_id = ''.join([pannel_num, '-detail-', str(cases_num)])
            hidden_id = ''.join([pannel_num, '-hidden-', str(cases_num)])
            cases_num += 1
            cases_packaged = ''.join([cases_packaged, '                   <tr class="{}">\n'.format(class_name)])
            for data in data_case[:-1]:
                if '详细' == data:
                    cases_packaged = ''.join(
                        [cases_packaged, REPORT_Table_Detail.format(detail_id, hidden_id, 'success', detail_id)])
                else:
                    cases_packaged = ''.join([cases_packaged, REPORT_Table_Data.format(data)])
            cases_packaged = ''.join([cases_packaged, '                   </tr>\n'])
            cases_packaged = ''.join(
                [cases_packaged, EXPORT_Detali_text.format(hidden_id, len(data_case) - 1, data_case[-1])])
        return cases_packaged

    def gen_html_report(self, html_template):
        export_label_title = ''
        for label_title in self.report_table_title:
            export_label_title = ''.join([export_label_title, REPORT_Title_Lable.format(label_title)])

        str_right_datas = ''
        str_untreated_cases = ''
        str_error_cases = ''
        str_all_cases = ''

        all_num = len(cases['all_cases'])
        right_num = len(cases['right_cases'])
        rate = '{:.2%}'.format(round(right_num / all_num, 5))

        for data_key in self.report_cases.keys():
            if 'right_cases' == data_key:
                str_right_datas = self.packaged_cases(export_label_title, data_key, 'success', 'panel1')
            elif 'untreaded_cases' == data_key:
                str_untreated_cases = self.packaged_cases(export_label_title, data_key, 'untreaded', 'panel3')
            elif 'error_cases' == data_key:
                str_error_cases = self.packaged_cases(export_label_title, data_key, 'error', 'panel2')
            else:
                str_all_cases = self.packaged_cases(export_label_title, data_key, 'all', 'panel0')
        export_label_datas = REPORT_TMPL_Lable.format(
            all_num,
            right_num,
            len(cases['error_cases']),
            len(cases['untreaded_cases']),
            str(str_all_cases[0:-1]),
            str(str_right_datas[0:-1]),
            str(str_error_cases[0:-1]),
            str(str_untreated_cases[0:-1])
        )
        # right_name_and_rate = '成功 ' + rate
        html_template = ''.join([html_template, export_label_datas])
        view = template(
            html_template,
            html_title=self.html_title,
            theme=self.pie_theme,
            start_time=self.test_info[0],
            end_time=self.test_info[1],
            used_time=self.test_info[2],
            sum_all_cases=self.test_info[3],
            sum_executed_cases=self.test_info[4],
            sum_untreaded_cases=self.test_info[5],
            right_sum=self.pie_sum_number['right_sum'],
            # rate='你好',
            error_sum=self.pie_sum_number['error_sum'],
            untreated_sum=self.pie_sum_number['untreated_sum']
        )
        return view


if __name__ == '__main__':
    detail = '''
        测试接口：
        是否通过：
        请求参数：
        接口响应：
    '''
    all_ = [['理财', '1', '登录', 'fdasfsadfsdfsdaf', 'post', 'Pass', '详细', detail],
            ['WEB理财', '2', '产品列表', 'list', 'post', 'Fail', '详细', '''(测试接口：www.baiduco\n是否通过：通过\n请求参数：\n接口响应：'fdsafsdafse!DOCTYPE html><!--STATUS OK-->\n\n\n    <html><head><meta \n')'''],
            ['理财', '3', '测试', 'login', 'dfsafefdfefsafe.com', '/', '详细',
             'resdfsaefdfffaaf:{fdsfdasfsdafaf:fdsfsd{fdsf}}']]
    right = [
        ['理财', '1', '登录', 'login', 'fdsaf.com', 'Pass', '详细', 'resdfsaefdfffaaf:{fdsfdasfsdafaf:fdsfsd{fdsf}}']]
    error = [['理财', '2', '产品列表', 'list', 'efdsfdsfsf.com', 'Fail', '详细', 'case_detail'],
             ['理财', '2', '产品列表', 'list', 'efdsfdsfsf.com', 'Fail', '详细', 'case_detail']]
    untreaded = [['WEB理财', '3', '测试', 'login', 'r23rfsdfew.con', '/', '详细', 'case_detail'],
                 ['WEB理财', '3', '测试', 'login', 'r23rfsdfew.con', '/', '详细', 'case_detail']]
    generate(all_, right, error, untreaded)
